#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright 2009  Olaf Merkert


# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see
# <http://www.gnu.org/licenses/>.


# README:

# This python script provides the functions to batch-check websites
# that provide pdf files for download.  It was developed for the use
# case of exercise sheets that are distributed via a university
# website.  Therefore, all pdfs are downloaded every time the program
# is run in order to detect updates (some professors have to make
# corrections or like to publish work-in-progress).  Consequently,
# rather a lot of bandwidth is used and depending on your network
# connection, things might take some time.  Of course, it would be
# much nicer if exercise sheets could be distributed by some atom or
# rss feeds.

# For usage information have a look at the example file.

from __future__ import with_statement
import re
from urlparse import urljoin, urlparse
from urllib import urlretrieve, urlopen
import os
import os.path as path
import hashlib


def dump_url(url):
    """put the contents served at the url in a string"""
    dump = ""
    try:
        stream = urlopen(url)
        dump = stream.read()
    finally:
        if stream: stream.close()
    return dump

def find_links(page_url, pattern):
    """find a list of urls using the regular expression pattern and
    anchor them at the page_url"""
    regexp = re.compile(pattern)
    contents = dump_url(page_url)
    return [urljoin(page_url, link_url)
            for link_url in re.findall(regexp, contents)]

def hash_file(local_file):
    """calculate a hash for the file"""
    m = hashlib.sha1()
    with open(local_file, 'r') as f:
        for line in f:
            m.update(line)
    return m.digest()

def file_exists(local_file):
    """check whether a local file exists"""
    return path.exists(local_file)

def download_file(remote_file, local_file):
    """copy a file from a remote location to a local place"""
    urlretrieve(remote_file, local_file)

def compare_files(previous, downloaded):
    """figure out whether the just downloaded file is different from
    the one we had before (if we had one)"""
    if not file_exists(previous):
        return True
    h1 = hash_file(previous)
    h2 = hash_file(downloaded)
    return h1 != h2

def extract_filename(file_url):
    """take an url and output the file part"""
    p = urlparse(file_url)
    return path.basename(p.path)

def check_exercise_page(title, url, target_dir):
    """find pdf links at url and put them into the target_dir"""
    updates = []
    for link in find_links(url, "[^\"']*\.pdf"):
        fname = extract_filename(link)
        prev = path.join(target_dir, fname)
        target = path.join(target_dir, fname + ".tmp")
        download_file(link, target)
        if compare_files(prev, target):
            os.rename(target, prev)
            updates.append("%s: %s" % (title, fname))
        else:
            os.remove(target)
    return updates

def walk_lectures(main_dir, lecture_list):
    updates = []
    for le in lecture_list:
        tdir = path.join(main_dir, le[0])
        if not path.exists(tdir):
            os.makedirs(tdir)
        updates.extend(check_exercise_page(
                le[0], le[1], tdir))
    return updates


                       
